#
# VFS for ruby - open anything with URI
# 
#   Copyright: Tatsuki Sugiura <sugi@nemui.org>
#   License: Ruby's original or GPL2
#   $LastChangedDate: 2003-11-10 23:46:31 +0900 (Mon, 10 Nov 2003) $
#   $LastChangedBy: sugi $
#

require 'vfs/dir'
require 'vfs/scheme/ftp/common'

module VFS;  module SchemeHandler;  module FTP;
  class Dir < VFS::Dir
    include VFS::SchemeHandler::FTP::Common

    def initialize(uri)
      super
      init_common
    end

    def mkdir
      ftp_request { |ftp|
        ftp.mkdir(uri.path)
      }
      self
    end

    def rmdir
      ftp_request { |ftp|
        ftp.rmdir(uri.path)
      }
      self
    end
    alias :delete :rmdir
    alias :unlink :rmdir
    
    def entries
      ls_la.map { |l|
        if /^\l\S+\s+\d+\s+\S+\s+\S+\s+\d+.{10,}\s+(\S+) ->/.match(l)
          $1.strip
        elsif /^\S+\s+\d+\s+\S+\s+\S+\s+\d+.{10,}\s+(\S+)/.match(l)
          $1.strip
        else
          nil
        end
      }.compact
    end

    private
    def ls_la(target = uri.path)
      ftp_request { |ftp|
        ftp.nlst("-la #{target.gsub(/([?\\* ])/,'\\\1')}")
      }
    end

  end # VFS::SchemeHandler::FTP::Dir
end; end; end

if $0 == __FILE__
  uri = "ftp://ftp.ring.gr.jp/"
  
  f = VFS::SchemeHandler::FTP::Dir.new(uri)
  p f.entries

  print "Password: "
  system("stty -echo") if STDIN.tty?
  pass = STDIN.gets.chomp
  system("stty echo") if STDIN.tty?
  puts
  uri = "ftp://sugi:#{URI.escape(pass)}@localhost:351/home/sugi"
  e = VFS::SchemeHandler::FTP::Dir.new(uri)
  p e.entries
  uri = "ftp://sugi:#{URI.escape(pass)}@localhost:351/home/sugi/tmp/ftpmkdir.t"
  w = VFS::SchemeHandler::FTP::Dir.new(uri)
  w.mkdir
  w.rmdir
end
