module Net
  class HTTP
    class Proppatch < HTTPRequest
      METHOD = 'PROPPATCH'
      REQUEST_HAS_BODY = true
      RESPONSE_HAS_BODY = true
    end
    class Lock < HTTPRequest
      METHOD = 'LOCK'
      REQUEST_HAS_BODY = true
      RESPONSE_HAS_BODY = true
    end
    class Unlock < HTTPRequest
      METHOD = 'UNLOCK'
      REQUEST_HAS_BODY = true
      RESPONSE_HAS_BODY = true
    end
    class Options < HTTPRequest
      METHOD = 'OPTIONS'
      REQUEST_HAS_BODY = false
      RESPONSE_HAS_BODY = false
    end
    class Propfind < HTTPRequest
      METHOD = 'PROPFIND'
      REQUEST_HAS_BODY = true
      RESPONSE_HAS_BODY = true
    end
    class Delete < HTTPRequest
      METHOD = 'DELETE'
      REQUEST_HAS_BODY = false
      RESPONSE_HAS_BODY = true
    end
    class Move < HTTPRequest
      METHOD = 'MOVE'
      REQUEST_HAS_BODY = false
      RESPONSE_HAS_BODY = true
    end
    class Copy < HTTPRequest
      METHOD = 'COPY'
      REQUEST_HAS_BODY = false
      RESPONSE_HAS_BODY = true
    end
    class Mkcol < HTTPRequest
      METHOD = 'MKCOL'
      REQUEST_HAS_BODY = true
      RESPONSE_HAS_BODY = true
    end
  end # HTTP
  
  class HTTP < Protocol
    def proppatch( path, body, initheader = nil ) 
      res = request(Proppatch.new(path, initheader), body)
      res.value unless @newimpl
      res
    end
    def lock( path, body, initheader = nil ) 
      res = request(Lock.new(path, initheader), body)
      res.value unless @newimpl
      res
    end
    def unlock( path, body, initheader = nil ) 
      res = request(Unlock.new(path, initheader), body)
      res.value unless @newimpl
      res
    end
    def options( path, initheader = nil ) 
      res = request(Options.new(path, initheader))
      res.value unless @newimpl
      res
    end
    def propfind( path, body = nil, initheader = { 'Depth' => '0' } )
      res = request(Propfind.new(path, initheader), body)
      res.value unless @newimpl
      res
    end
    def delete( path, initheader = { 'Depth' => 'Infinity' } )
      res = request(Delete.new(path, initheader))
      res.value unless @newimpl
      res
    end
    def move( path, initheader = nil )
      res = request(Move.new(path, initheader))
      res.value unless @newimpl
      res
    end
    def copy( path, initheader = nil )
      res = request(Copy.new(path, initheader))
      res.value unless @newimpl
      res
    end
    def mkcol( path, body = nil, initheader = nil)
      res = request(Mkcol.new(path, initheader), body)
      res.value unless @newimpl
      res
    end
  end # HTTP < Protocol
end # Net

